#if NET46 || NETSTANDARD20
#define SERIALIZATION
#endif

using System;
using System.Diagnostics;
#if SERIALIZATION
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
#endif
using NUnit.Framework;

namespace Smdn {
  internal static partial class TestUtils {
    internal static partial class Assert {
      public static void IsSerializableBinaryFormat<TSerializable>(TSerializable obj)
      /*where TSerializable : ISerializable*/
      {
        IsSerializableBinaryFormat(obj, null);
      }

      public static void IsSerializableBinaryFormat<TSerializable>(TSerializable obj,
                                                                   Action<TSerializable> testDeserializedObject)
      /*where TSerializable : ISerializable*/
      {
#if SERIALIZATION
      var serializeFormatter = new BinaryFormatter();

      using (var stream = new MemoryStream()) {
        serializeFormatter.Serialize(stream, obj);

        stream.Position = 0L;

        var deserializeFormatter = new BinaryFormatter();
        var deserialized = deserializeFormatter.Deserialize(stream);

        NUnit.Framework.Assert.IsNotNull(deserialized);
        NUnit.Framework.Assert.AreNotSame(obj, deserialized);
        NUnit.Framework.Assert.IsInstanceOf<TSerializable>(deserialized);

        if (testDeserializedObject != null)
          testDeserializedObject((TSerializable)deserialized);
      }
#else
        // do nothing
#endif
      }

      //private static readonly TimeSpan mergin = TimeSpan.FromTicks(Stopwatch.Frequency);
      private static readonly TimeSpan mergin = TimeSpan.FromMilliseconds(20);

      public static void Elapses(TimeSpan expectedSpan, TestDelegate code)
      {
        var sw = Stopwatch.StartNew();

        code();

        sw.Stop();

        NUnit.Framework.Assert.GreaterOrEqual(sw.Elapsed + mergin, expectedSpan);
      }

      public static void Elapses(TimeSpan expectedSpanRangeMin, TimeSpan expectedSpanRangeMax, TestDelegate code)
      {
        var sw = Stopwatch.StartNew();

        code();

        sw.Stop();

        NUnit.Framework.Assert.GreaterOrEqual(sw.Elapsed + mergin, expectedSpanRangeMin);
        NUnit.Framework.Assert.LessOrEqual(sw.Elapsed - mergin, expectedSpanRangeMax);
      }

      public static void NotElapse(TimeSpan expectedSpan, TestDelegate code)
      {
        var sw = Stopwatch.StartNew();

        code();

        sw.Stop();

        NUnit.Framework.Assert.LessOrEqual(sw.Elapsed - mergin, expectedSpan);
      }

      public static TException ThrowsOrAggregates<TException>(TestDelegate code)
        where TException : Exception
      {
        try {
          code();

          NUnit.Framework.Assert.Fail("expected exception {0} not thrown", typeof(TException).FullName);

          return null;
        }
        catch (AssertionException) {
          throw;
        }
        catch (Exception ex) {
          var aggregateException = ex as AggregateException;

          if (aggregateException == null) {
            NUnit.Framework.Assert.IsInstanceOf<TException>(ex);

            return ex as TException;
          }
          else {
            NUnit.Framework.Assert.IsInstanceOf<TException>(aggregateException.Flatten().InnerException);

            return aggregateException.InnerException as TException;
          }
        }
      }
    } // Assert

    public static void Repeat(int count, Action action)
    {
      for (var i = 0; i < count; i++) {
        action();
      }
    }
  }
}
